% Modal Interferometer Simulation Tool (MIST) - Gabriele Vajente (April 2010)
%
% Compute normalized error signals to lock dual recycled interferometer
%
% [eprcl, ecarm, emich, edarm, esrcl] = error_signals_norm(itf, lock)
%
%       itf  = interferometer structure
%       lock = structure containing lock parameters 
%               gcarm, gprcl, gmich, gdarm, gsrcl: loop gains 
%                   (zero means not locked)
%               tolcarm, toldarm, tolmich, tolprcl, tolsrc: maximum tolerances
%                                                     for error signals
%
% Error signals used
%
% DARM  ASY_DC_00 - 8e-4
% CARM  PRC_AC1_P / SQRT(PRC_POW_USB)
% PRCL  REF_AC3_P
% MICH  PRC_AC2_Q
% SRCL  PRC_AC2_P

function [eprcl, ecarm, emich, edarm, esrcl] = error_signals_norm(itf, lock)

    % compute fields
    [crc, cdf, cref] = dualrecycled_fields_forlock(itf, 0);
    [u1rc, u1df, u1ref] = dualrecycled_fields_forlock(itf,  itf.fmod1);
    [l1rc, l1df, l1ref] = dualrecycled_fields_forlock(itf, -itf.fmod1);
    [u2rc, u2df, u2ref] = dualrecycled_fields_forlock(itf,  itf.fmod2);
    [l2rc, l2df, l2ref] = dualrecycled_fields_forlock(itf, -itf.fmod2);
    [u3rc, u3df, u3ref] = dualrecycled_fields_forlock(itf,  8.36e6);
    [l3rc, l3df, l3ref] = dualrecycled_fields_forlock(itf, -8.36e6);
    
    % compute AC signals
    [prc_p, prc_q]   = ac(crc(:,1), u1rc(:,1), l1rc(:,1), 0.1, 90);
    %[ref_p, ref_q]   = ac(cref(:,1), u1ref(:,1), l1ref(:,1), 0.1, 90);
    [prc_p2, prc_q2] = ac(crc(:,1), u2rc(:,1), l2rc(:,1), 0.1, 90);
    [ref_p3, ref_q3]   = ac(cref(:,1), u3ref(:,1), l3ref(:,1), 0.1, 90);
    %[df_p2, df_q2] = ac(cdf(:,1), u2df(:,1), l2df(:,1), 0.1, 0);
    
    % build error signals
    eprcl = lock.gprcl * ref_p3;
    ecarm = lock.gcarm * prc_p ./ abs(u1rc(1,1));
    edarm = lock.gdarm * (abs(cdf(1,1))^2 - 8e-4);
    emich = lock.gmich * prc_q2;
    esrcl = lock.gsrcl * prc_p2;
    
