% Modal Interferometer Simulation Tool (MIST) - Gabriele Vajente (April 2010)
%
% Return the transmission matrix for a lens using a given mode basis.
% This function is optimized for spherical lenses and even modes only.
% 
% M = lens_optimized_even_modes(f, w, nM, rlens, npt)
%
%     f     = lens focal
%     w     = beam size at lens
%     nM    = maximum index of even modes
%     rlens = lens radius
%     npt   = number of points for numerical integration

function M = lens_optimized_even_modes(f, w, nM, rlens, npt)
    tic;
    k = 2*pi / 1.064e-6;

    mm = list_even_modes(nM);
    mx = 0:2:nM;
    N = length(mx);
    
    % build integration variables
    x = linspace(-rlens, rlens, npt);
    dx = x(2) - x(1);
    
    % build integrand
    x2 = x.^2;
    Z = exp(2j*k*x2 / (4*f));
    g = exp(-2*x2/w^2);
    
    % compute single variable integrals
    I = zeros(N,N);
    for m=1:N
        for p=1:N
            norm = 1/sqrt(2^(mx(m)+mx(p)) * factorial(mx(m)) * ...
                    factorial(mx(p)));
            norm = norm * sqrt(2 / (pi * w^2)) * dx;
            
            hm = polyval(HermitePoly(mx(m)), sqrt(2) * x / w);
            hp = polyval(HermitePoly(mx(p)), sqrt(2) * x / w);
            
            I(m,p) = norm * sum(hm .* hp .* Z .* g);
        end
    end
    
    % build full matrix
    N = length(mm);   
    M = zeros(N,N);
      
    for i=1:N
        for j=1:N
            M(i,j) = I(mm(i,1)/2+1, mm(j,1)/2+1) * I(mm(i,2)/2+1, mm(j,2)/2+1);
        end
    end
           
    el = toc;
    disp(['Elapsed time: ' num2str(el) ' s'])     
