% Modal Interferometer Simulation Tool (MIST) - Gabriele Vajente (April 2010)
%
% Lock a dual recycled interferometer using real error signals
%
% function [zcarm, zprcl, zmich, zdarm, zsrcl, niter] = 
%                           lockdual_norm(itf, lock, nitermax, g)
%
%     itf  = interferometer structure
%     lock = structure containing lock parameters 
%         gcarm, gprcl, gmich, gdarm, gsrcl: loop gains (zero means not locked)
%         tolcarm, toldarm, tolmich, tolprcl, tolsrc: maximum tolerances
%                                                     for error signals
%     nitermax = maximum number of iterations
%     g        = loop integrator gain 
%
%     zcarm, zprcl, zmich, zdarm, zsrcl = final locked positions
%     niter                             = number of iterations perfomed
%


function [zcarm, zprcl, zmich, zdarm, zsrcl, niter] = lockdual_norm(itf, lock, nitermax, g)
    
    zdarm = zeros(nitermax,1);
    zcarm = zeros(nitermax,1);
    zprcl = zeros(nitermax,1);
    zmich = zeros(nitermax,1);
    zsrcl = zeros(nitermax,1);
    edarm = zeros(nitermax,1);
    ecarm = zeros(nitermax,1);
    eprcl = zeros(nitermax,1);
    emich = zeros(nitermax,1);
    esrcl = zeros(nitermax,1);

    % init corrections
    zdarm(1) = itf.DARM;
    zcarm(1) = itf.CARM;
    zmich(1) = itf.MICH;
    zprcl(1) = itf.PRCL;
    zsrcl(1) = itf.phi_src;
    
    % lock
    for i=1:nitermax
        % compute error signals
        if lock.gdarm ~= 0
            itf.DARM    = zdarm(i);
        end
        if lock.gcarm ~= 0
            itf.CARM    = zcarm(i);
        end
        if lock.gmich ~= 0 
            itf.MICH    = zmich(i);
        end
        if lock.gprcl ~= 0 
            itf.PRCL    = zprcl(i);
        end
        if lock.gsrcl ~= 0 
            itf.phi_src = zsrcl(i);
        end
        
        [eprcl(i), ecarm(i), emich(i), edarm(i), esrcl(i)] = error_signals_norm(itf, lock);
        
        locked = ((abs(eprcl(i)) < lock.tolprcl ) | (lock.gprcl == 0)) & ...
                 ((abs(ecarm(i)) < lock.tolcarm ) | (lock.gcarm == 0)) & ...
                 ((abs(emich(i)) < lock.tolmich ) | (lock.gmich == 0)) & ...
                 ((abs(edarm(i)) < lock.toldarm ) | (lock.gdarm == 0)) & ...
                 ((abs(esrcl(i)) < lock.tolsrcl ) | (lock.gsrcl == 0));
         
        if locked
            break;
        end
        
        if (i == 1)
            if lock.gdarm ~= 0
                zdarm(i+1) = zdarm(i) - edarm(i) * g;
            end
            if lock.gcarm ~= 0
                zcarm(i+1) = zcarm(i) - ecarm(i) * g;
            end
            if lock.gcarm ~= 0
                zprcl(i+1) = zprcl(i) - eprcl(i) * g;
            end
            if lock.gmich ~= 0
                zmich(i+1) = zmich(i) - emich(i) * g;
            end
            if lock.gsrcl ~= 0
                zsrcl(i+1) = zsrcl(i) - esrcl(i) * g;
            end
        else
            if lock.gdarm ~= 0
                zdarm(i+1) = zdarm(i) - (edarm(i) + edarm(i-1)) * g;
            end
            if lock.gcarm ~= 0
                zcarm(i+1) = zcarm(i) - (ecarm(i) + ecarm(i-1)) * g;
            end
            if lock.gprcl ~= 0
                zprcl(i+1) = zprcl(i) - (eprcl(i) + eprcl(i-1)) * g;
            end
            if lock.gmich ~= 0
                zmich(i+1) = zmich(i) - (emich(i) + emich(i-1)) * g;
            end
            if lock.gsrcl ~= 0
                zsrcl(i+1) = zsrcl(i) - (esrcl(i) + esrcl(i-1)) * g;
            end
        end
   end  
    
   zdarm = zdarm(i);
   zcarm = zcarm(i);
   zprcl = zprcl(i);
   zmich = zmich(i);
   zsrcl = zsrcl(i);
   zsrcl = itf.phi_src;
   niter = i;
    
