% Modal Interferometer Simulation Tool (MIST) - Gabriele Vajente (April 2010)
%
% Return the reflection matrix for a mirror with mis-matched
% radius of curvature and a tilt, using given gaussian basis for input
% and output beams. Astigmatism included
% 
% M = mirror_reflection_qxy(Rc, theta, qiy, qix, qox, qoy, modes, rmir, npt, thr)
%
%     Rc     = mirror radius of curvature
%     theta  = mirror tilt
%     qix, qox = beam parameters at input and output (x direction)
%     qiy, qoy = beam parameters at input and output (y direction)
%     modes  = list of all modes to be used in [n,m] format
%     rm     = mirror radius in units of spot size
%     npt    = number of points for numerical integration
%     thr    = below this values coupling coefficients are set to zero

function M = mirror_reflection_qxy(Rc, aoi, theta, qix, qiy, qox, qoy, modes, rm, npt, thr)
    k = 2*pi / 1.064e-6;

    % input and output beam parameters
    Rix = 1/real(1/qix);
    wix  = sqrt( -1.064e-6/(pi*imag(1/qix)));
    Rox = 1/real(1/qox);
    wox  = sqrt( -1.064e-6/(pi*imag(1/qox)));
      
    Riy = 1/real(1/qiy);
    wiy  = sqrt( -1.064e-6/(pi*imag(1/qiy)));
    Roy = 1/real(1/qoy);
    woy  = sqrt( -1.064e-6/(pi*imag(1/qoy)));
    
    % use average spot size
    rmir = rm * (wix*wox*wiy*woy)^0.25;    
    
    % build integration grid
    [x,y] = meshgrid(linspace(-rmir,rmir,npt),linspace(-rmir,rmir,npt));
    dxdy = (2*rmir / (npt - 1))^2;
    
    % build phase map function
    x2 = x.^2;
    y2 = y.^2;
    r2 = x2 + y2;
    
    RCx = Rc * cos(aoi*pi/180);
    RCy = Rc / cos(aoi*pi/180);
    
    % build exponential integrand
    %g = exp( -x2 * (1/wix^2 + 1/wox^2) -y2 * (1/wiy^2 + 1/woy^2) - 2j*k*theta*x);
    g = exp( -x2 * (1/wix^2 + 1/wox^2 + 0.5j*k/Rix - 0.5j*k/Rox - 1j*k/RCx) ...
             -y2 * (1/wiy^2 + 1/woy^2 + 0.5j*k/Riy - 0.5j*k/Roy - 1j*k/RCy) ...
             - 2j*k*theta*x);
   
    % build mask for mirror radius
    msk = r2 < (rmir*rmir); 
    
    % compute matrix for each mode
    N = size(modes,1);
    M = zeros(N,N);
    for i=1:N
        for j=1:N
            m = modes(i,1);
            n = modes(i,2);
            p = modes(j,1);
            q = modes(j,2);
        
            norm = 1/sqrt(2^(n+m+p+q) * factorial(n) * factorial(m) * ...
                    factorial(p) * factorial(q));
            norm = norm * 2 / (pi * sqrt(wix * wox * wiy * woy)) * dxdy;
                        
            % compute hermite polynomials (in correct variables)
            hm = polyval(HermitePoly(m), sqrt(2) * x / wox);
            hn = polyval(HermitePoly(n), sqrt(2) * y / woy);
            hp = polyval(HermitePoly(p), sqrt(2) * x / wix);
            hq = polyval(HermitePoly(q), sqrt(2) * y / wiy);
            
            % integrate
            M(i,j) = norm * sum(sum(msk.* hp .* hq .* hm .* hn .* g));
            
            % kill small elements
            if abs(M(i,j)) < thr
                M(i,j) = 0;
            end
        end
    end
