% Modal Interferometer Simulation Tool (MIST) - Gabriele Vajente (April 2010)
%
% Return the reflection matrix for a mirror with mis-matched
% radius of curvature, using a given mode basis
% 
% M = mirror_transmission_map(mapfile, s, c, w, modes)
%
%     mapfile = phase map filename
%     c       = indexes of optical center
%     s       = step size of map
%     w       = beam size at mirror
%     modes   = list of all modes to be used in [n,m] format

function M = mirror_transmission_map(mapfile, s, c, w, modes)
    tic;
    k = 2*pi / 1.064e-6;
    n = 1.45;

    % load map file
    map = load(mapfile);
    map = 1e-9 * map;
    x = ([1:size(map,2)] - c(2)) * s;
    y = ([1:size(map,1)] - c(1)) * s;
    
    [x,y] = meshgrid(x,y);
    dxdy = s*s;
        
    % build Gaussian weight
    r2 = x.^2 + y.^2;
    g = exp(-2*r2/w^2);
    
    % build mask for mirror radius
    
    % compute matrix for each mode
    N = size(modes,1);
    M = zeros(N,N);
    h = waitbar(0, 'Computing');
    for i=1:N
        for j=1:N
            m = modes(i,1);
            n = modes(i,2);
            p = modes(j,1);
            q = modes(j,2);
        
            norm = 1/sqrt(2^(n+m+p+q) * factorial(n) * factorial(m) * ...
                    factorial(p) * factorial(q));
            norm = norm * 2 / (pi * w^2) * dxdy;
                        
            % compute hermite polynomials (in correct variables)
            hm = polyval(HermitePoly(m), sqrt(2) * x / w);
            hn = polyval(HermitePoly(n), sqrt(2) * y / w);
            hp = polyval(HermitePoly(p), sqrt(2) * x / w);
            hq = polyval(HermitePoly(q), sqrt(2) * y / w);
            
            % integrate
            M(i,j) = norm * sum(sum(hp .* hq .* hm .* hn .* exp(1j*k*(n-1)*map) .* g));
            
            el = toc;
            ix = (i-1)*N + j;
            eta = floor((el / ix * (N*N - ix)));
            waitbar(ix/(N*N), h, ['Dual recycled locked (', num2str(floor(100*ix/(N*N))), '% ETA = ', num2str(eta), 's)']);
    
        end
    end
            
    el = toc;
    disp(['Elapsed time: ' num2str(el) ' s'])
    close(h)
