% Modal Interferometer Simulation Tool (MIST) - Gabriele Vajente (April 2010)
%
% Compute operator for astigmatic power recycling cavity. All astigmatism and
% Gouy phases are coded inside the recycling mirror operator (reflection
% and transmission).
%
% Recycling mirror are assumed to be perfectly matched, but can be tilted.
%
% function [R,T, RC, w] = ndrc_power_recycling(rec, cav, modes, theta1, theta2, theta3)
%
% rec   = structure for describing recyling cavity
%         rec.L1, rec.L2, rec.L3, rec.R2, rec.R3, rec.alpha
% cav   = structure describing arm cavity
% modes = list of modes to be used
% theta1, theta2, theta3 = tilts of PRM1, PRM2, PRM3
%
% R  = recycling mirror operator (reflection from BS to BS
% T  = recycling mirror operator (transmission from input to BS)
% RC = PRM radius of curvature (spherical and optimally matched)
% w  = beam size on PRM1



function [R, T, R1, w] = ndrc_power_recycling(rec, cav, modes, theta1, theta2, theta3)
    global  npt thr 
    
    M = length(modes);
    
    %%%%%%%%%%%% COMPUTE PRM1 PARAMETERS
        
    % beam parameter at cavity reflection
    q0 = cav.L1 + 1j * cav.z0;
    
    % take into account (diverging) lens due to input mirror substrate
    n = 1.44963;
    MLI = [1, 0; (n-1)/cav.R1 1];
    
    % compute mode at input mirror exit
    q1 = ABCD(MLI, q0);
    
    % propagate gaussian parameter to PRM1 and compute Gouy phase
    % in the non-astigmatic basis
    ML1 = [1 rec.L1; 0 1];
    ML2 = [1 rec.L2; 0 1];
    ML3 = [1 rec.L3; 0 1];
    MR3 = [1 0; -2/rec.R3 1];
    MR2 = [1 0; -2/rec.R2 1];
    
    qprm3in_0  = ABCD(ML3, q1);
    qprm3out_0 = ABCD(MR3, qprm3in_0);
    qprm2in_0  = ABCD(ML2, qprm3out_0);
    qprm2out_0 = ABCD(MR2, qprm2in_0);
    qprm1in_0  = ABCD(ML1, qprm2out_0);
    
    % compute beam parameters at PRM1
    R1 = 1/real(1/qprm1in_0);
    w  = sqrt( -1.064e-6/(pi*imag(1/qprm1in_0)));
    
    %%%%%%%%%%%% PROPAGATE CAVITY BEAM BASIS INSIDE PRC TOWARD PRM1
           
    % propagate gaussian parameter to PRM1 and compute Gouy phases
    % in the astigmatic basis
    a = rec.alpha / 180 * pi;
    
    MR3t = [1 0; -2/(rec.R3*cos(a)) 1];
    MR3s = [1 0; -2*cos(a)/rec.R3   1];
    MR2t = [1 0; -2/(rec.R2*cos(a)) 1];
    MR2s = [1 0; -2*cos(a)/rec.R2   1];
     
    qprm3in_t  = ABCD(ML3, q1);
    qprm3out_t = ABCD(MR3t, qprm3in_t);
    qprm2in_t  = ABCD(ML2, qprm3out_t);
    qprm2out_t = ABCD(MR2t, qprm2in_t);
    qprm1in_t  = ABCD(ML1, qprm2out_t);
    phig_t_i3 = atan(real(qprm3in_t)/imag(qprm3in_t)) - atan(real(q1)/imag(q1));
    phig_t_23 = atan(real(qprm2in_t)/imag(qprm2in_t)) - atan(real(qprm3out_t)/imag(qprm3out_t));
    phig_t_12 = atan(real(qprm1in_t)/imag(qprm1in_t)) - atan(real(qprm2out_t)/imag(qprm2out_t));
         
    qprm3in_s  = ABCD(ML3, q1);
    qprm3out_s = ABCD(MR3s, qprm3in_s);
    qprm2in_s  = ABCD(ML2, qprm3out_s);
    qprm2out_s = ABCD(MR2s, qprm2in_s);
    qprm1in_s  = ABCD(ML1, qprm2out_s);
    phig_s_i3 = atan(real(qprm3in_s)/imag(qprm3in_s)) - atan(real(q1)/imag(q1));
    phig_s_23 = atan(real(qprm2in_s)/imag(qprm2in_s)) - atan(real(qprm3out_s)/imag(qprm3out_s));
    phig_s_12 = atan(real(qprm1in_s)/imag(qprm1in_s)) - atan(real(qprm2out_s)/imag(qprm2out_s));
             
    %%%%%%%%%%%% REFLECT BEAM ON PRM1 AND BACK TO INPUT MIRROR
    
    MR1  = [1 0; -2/R1 1];
      
    Qprm1out_t = ABCD(MR1, qprm1in_t);   % backward modes are capital Q
    Qprm2in_t  = ABCD(ML1, Qprm1out_t);
    Qprm2out_t = ABCD(MR2t, Qprm2in_t);
    Qprm3in_t  = ABCD(ML2, Qprm2out_t);
    Qprm3out_t = ABCD(MR3t, Qprm3in_t);
    Q1_t       = ABCD(ML3, Qprm3out_t);
    Phig_t_12 = atan(real(Qprm2in_t)/imag(Qprm2in_t)) - atan(real(Qprm1out_t)/imag(Qprm1out_t));
    Phig_t_23 = atan(real(Qprm3in_t)/imag(Qprm3in_t)) - atan(real(Qprm2out_t)/imag(Qprm2out_t));
    Phig_t_i3 = atan(real(Q1_t)/imag(Q1_t)) - atan(real(Qprm3out_t)/imag(Qprm3out_t));
    
    Qprm1out_s = ABCD(MR1, qprm1in_s);   % backward modes are capital Q
    Qprm2in_s  = ABCD(ML1, Qprm1out_s);
    Qprm2out_s = ABCD(MR2s, Qprm2in_s);
    Qprm3in_s  = ABCD(ML2, Qprm2out_s);
    Qprm3out_s = ABCD(MR3s, Qprm3in_s);
    Q1_s       = ABCD(ML3, Qprm3out_s);
    Phig_s_12 = atan(real(Qprm2in_s)/imag(Qprm2in_s)) - atan(real(Qprm1out_s)/imag(Qprm1out_s));
    Phig_s_23 = atan(real(Qprm3in_s)/imag(Qprm3in_s)) - atan(real(Qprm2out_s)/imag(Qprm2out_s));
    Phig_s_i3 = atan(real(Q1_s)/imag(Q1_s)) - atan(real(Qprm3out_s)/imag(Qprm3out_s));
     
    
    %%%%%%%%%%%% COMPUTE ALL OPERATORS FROM INPUT MIRROR BACK TO INPUT MIRROR
    
    
    % propagation from input mirror to PRM3
    Pi3 = diag(1j * exp(1j * (modes(:,1) * phig_s_i3 + modes(:,2) * phig_t_i3)));
    % reflection on PRM3
    Mprm3 = mirror_reflection_qxy(rec.R3, rec.alpha, theta3, qprm3in_t, qprm3in_s, qprm3out_t, qprm3out_s, ...
        modes, 5, npt, thr);
    % propagation from PRM3 to PRM2
    P32 = diag(1j * exp(1j * (modes(:,1) * phig_s_23 + modes(:,2) * phig_t_23)));
    % reflection on PRM2
    Mprm2 = mirror_reflection_qxy(rec.R2, rec.alpha, theta2, qprm2in_t, qprm2in_s, qprm2out_t, qprm2out_s, ...
        modes, 5, npt, thr);
    % propagation from PRM2 to PRM1
    P21 = diag(1j * exp(1j * (modes(:,1) * phig_s_12 + modes(:,2) * phig_t_12)));
    % reflection on PRM1
    Mprm1 = mirror_reflection_qxy(R1, 0, theta1, qprm1in_t, qprm1in_s, Qprm1out_t, Qprm1out_s, ...
        modes, 5, npt, thr);
    % propagation from PRM1 to PRM2 
    P12 = diag(1j * exp(1j * (modes(:,1) * Phig_s_12 + modes(:,2) * Phig_t_12)));
    % reflection on PRM2
    Mprm2b = mirror_reflection_qxy(rec.R2, rec.alpha, theta2, Qprm2in_t, Qprm2in_s, Qprm2out_t, Qprm2out_s, ...
        modes, 5, npt, thr);
    % propagation from PRM2 to PRM3
    P23 = diag(1j * exp(1j * (modes(:,1) * Phig_s_23 + modes(:,2) * Phig_t_23)));
    % reflection on PRM3
    Mprm3b = mirror_reflection_qxy(rec.R3, rec.alpha, theta3, Qprm3in_t, Qprm3in_s, Qprm3out_t, Qprm3out_s, ...
        modes, 5, npt, thr);
    % propagation from PRM3 to input
    P3i = diag(1j * exp(1j * (modes(:,1) * Phig_s_i3 + modes(:,2) * Phig_t_i3)));
    
    %%%%%%%%%%%% PROJECTION BACK TO ROUND CAVITY BASIS
    
    MC = coupling_matrix(Q1_s, Q1_t, -conj(q1), -conj(q1), modes, 5, npt, thr);
    
    
    %%%%%%%%%%%% PUT EVERYTHING TOGHETER 
    %%%%%%%%%%%% (REFLECTION OPERATOR AS SEEN INSIDE ITF)
    
    % reflection from input to input
    R = MC * P3i * Mprm3b * P23 * Mprm2b * P12 * Mprm1 * P21 * Mprm2 * P32 * Mprm3 * Pi3;
    % rescale to maintain 00 resonant
    R = R / exp(1j * angle(R(1,1)));
  
    %%%%%%%%%%%% PROPAGATE BEAM BASIS AT INPUT TO ARM CAVITY
    
    % input beam matched to PRM1
    Qprm1out_0  = -conj(qprm1in_0);
    % propagate from PRM1 to PRM2
    Qprm2in_0 = ABCD(ML1, Qprm1out_0);
    Phig_0_12 = atan(real(Qprm2in_0)/imag(Qprm2in_0)) - atan(real(Qprm1out_0)/imag(Qprm1out_0));
    
    % reflection on PRM2
    Qprm2out_t = ABCD(MR2t, Qprm2in_0);
    % propagation from PRM2 to PRM3
    Qprm3in_t = ABCD(ML2, Qprm2out_t);
    % reflection on PRM3
    Qprm3out_t = ABCD(MR3t, Qprm3in_t);
    % propagation from PRM3 to input
    Qin_t = ABCD(ML3, Qprm3out_t);
    
    Phig_t_23 = atan(real(Qprm3in_t)/imag(Qprm3in_t)) - atan(real(Qprm2out_t)/imag(Qprm2out_t));
    Phig_t_3i = atan(real(Qin_t)/imag(Qin_t)) - atan(real(Qprm3out_t)/imag(Qprm3out_t));
    
    % reflection on PRM2
    Qprm2out_s = ABCD(MR2s, Qprm2in_0);
    % propagation from PRM2 to PRM3
    Qprm3in_s = ABCD(ML2, Qprm2out_s);
    % reflection on PRM3
    Qprm3out_s = ABCD(MR3s, Qprm3in_s);
    % propagation from PRM3 to input
    Qin_s = ABCD(ML3, Qprm3out_s);
    
    Phig_s_23 = atan(real(Qprm3in_s)/imag(Qprm3in_s)) - atan(real(Qprm2out_s)/imag(Qprm2out_s));
    Phig_s_3i = atan(real(Qin_s)/imag(Qin_s)) - atan(real(Qprm3out_s)/imag(Qprm3out_s));
    
    %%%%%%%%%%%% COMPUTE ALL OPERATORS FROM INPUT BEAM TO INPUT MIRROR
    
    % propagation from PRM1 to PRM2
    P12 = diag(1j * exp(1j * (modes(:,1) * Phig_0_12 + modes(:,2) * Phig_0_12)));
    % reflection on PRM2
    Mprm2 = mirror_reflection_qxy(rec.R2, rec.alpha, theta2, Qprm2in_0, Qprm2in_0, Qprm2out_t, Qprm2out_s, ...
        modes, 5, npt, thr);
    % propagation from PRM2 to PRM3
    P23 = diag(1j * exp(1j * (modes(:,1) * Phig_s_23 + modes(:,2) * Phig_t_23)));
    % reflection on PRM3
    Mprm3 = mirror_reflection_qxy(rec.R3, rec.alpha, theta3, Qprm3in_t, Qprm3in_s, Qprm3out_t, Qprm3out_s, ...
        modes, 5, npt, thr);
    % propagation from PRM3 to input mirror
    P3i = diag(1j * exp(1j * (modes(:,1) * Phig_s_i3 + modes(:,2) * Phig_t_i3)));
    
    %%%%%%%%%%%% PUT EVERYTHING TOGHETER 
    %%%%%%%%%%%% (TRANSMISSION OPERATOR AS SEEN BY INPUT BEAM)
    T = P3i * Mprm3 * P23 * Mprm2 * P12;
    
    
    
    
