% Modal Interferometer Simulation Tool (MIST) - Gabriele Vajente (April 2010)
%
% Return the transmission matrix for the residual lens computed from
% TCS phase maps (read from file)
% 
% M = tcs_lens(filename, dx, w, modes, r, thr)
%
%     filename = phase map datafile
%     dx       = transversal dimension
%     w        = beam size at lens
%     modes    = list of all modes to be used in [n,m] format
%     r        = radius of lens t be used
%     thr      = threshold

function M = tcs_lens(filename, dx, w, modes, r, thr)
    %tic;
    k = 2*pi / 1.064e-6;

    % load phase map
    pm = load(filename);
    pm(find(isnan(pm))) = 0;
    npt = sqrt(length(pm));
    
    % build integration grid
    [x,y] = meshgrid(linspace(-dx/2,dx/2,npt),linspace(-dx/2,dx/2,npt));
    dxdy = (2*dx/2 / (npt - 1))^2;
    
    % build Gaussian weight
    r2 = x.^2 + y.^2;
    g = exp(-2*r2/w^2);
    
    % build phase map
    PM = reshape(pm, npt, npt);
    
    % build mask for mirror radius
    msk = r2 < (r*r); 
    
    % compute matrix for each mode
    N = length(modes);
    M = zeros(N,N);
    for i=1:N
        for j=1:N
            m = modes(i,1);
            n = modes(i,2);
            p = modes(j,1);
            q = modes(j,2);
        
            norm = 1/sqrt(2^(n+m+p+q) * factorial(n) * factorial(m) * ...
                    factorial(p) * factorial(q));
            norm = norm * 2 / (pi * w^2) * dxdy;
                        
            % compute hermite polynomials (in correct variables)
            hm = polyval(HermitePoly(m), sqrt(2) * x / w);
            hn = polyval(HermitePoly(n), sqrt(2) * y / w);
            hp = polyval(HermitePoly(p), sqrt(2) * x / w);
            hq = polyval(HermitePoly(q), sqrt(2) * y / w);
            
            % integrate
            M(i,j) = norm * sum(sum(msk.* hp .* hq .* hm .* hn .* exp(1j*PM) .* g));
            
            % kill small elements
            if abs(M(i,j)) < thr
                M(i,j) = 0;
            end
        end
    end
            
    %el = toc;
    %disp(['Elapsed time: ' num2str(el) ' s'])     
